"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _immutable = require("immutable");

var actionTypes = _interopRequireWildcard(require("../constants/action-types"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj["default"] = obj; return newObj; } }

var initialState = (0, _immutable.Map)();

var queries = function queries() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
  var action = arguments.length > 1 ? arguments[1] : undefined;

  switch (action.type) {
    case actionTypes.RESET:
      {
        return initialState;
      }

    case actionTypes.MUTATE_START:
    case actionTypes.REQUEST_START:
      {
        var query = (0, _immutable.Map)({
          url: action.url,
          isFinished: false,
          isPending: true,
          isMutation: action.type === actionTypes.MUTATE_START,
          queryCount: state.get(action.queryKey) ? state.getIn([action.queryKey, 'queryCount']) + 1 : 1
        });
        return state.setIn([action.queryKey], query);
      }

    case actionTypes.REQUEST_SUCCESS:
    case actionTypes.MUTATE_FAILURE:
    case actionTypes.MUTATE_SUCCESS:
    case actionTypes.REQUEST_FAILURE:
      {
        var _query = (0, _immutable.Map)({
          isFinished: true,
          isPending: false,
          lastUpdated: action.time,
          status: action.status,
          headers: (0, _immutable.fromJS)(action.responseHeaders)
        });

        return state.mergeIn([action.queryKey], _query);
      }

    case actionTypes.CANCEL_QUERY:
      {
        if (action.queryKey && state.getIn([action.queryKey, 'isPending'], false)) {
          // Make sure request is actually pending
          var _query2 = (0, _immutable.Map)({
            isFinished: true,
            isPending: false,
            status: 0
          });

          return state.mergeIn([action.queryKey], _query2);
        }

        return state;
      }

    default:
      {
        return state;
      }
  }
};

var _default = queries;
exports["default"] = _default;