// @flow

import { Map, fromJS } from 'immutable';
import type { Update, Entities, OptimisticUpdate, Rollback } from '../types';

export const updateEntities = (
  update: Update = {},
  entities: Entities = Map(),
  transformed: Entities = Map(),
): Entities => {
  // If update is not supplied, then no change to entities will be made

  return Object.keys(update || {}).reduce((accum, key) => {
    return accum.set(key, update[key](fromJS(entities.get(key)), fromJS(transformed.get(key))));
  }, Map());
};

export const optimisticUpdateEntities = (
  optimisticUpdate: OptimisticUpdate = {},
  entities: Entities = Map(),
): Entities => {
  return Object.keys(optimisticUpdate).reduce((accum, key) => {
    if (optimisticUpdate[key]) {
      return accum.set(key, optimisticUpdate[key](fromJS(entities.get(key))));
    }
    return accum.set(key, entities.get(key));
  }, Map());
};

export const rollbackEntities = (
  rollback: Rollback = {},
  initialEntities: Entities = Map(),
  entities: Entities = Map(),
): Entities => {
  return Object.keys(initialEntities.toObject()).reduce((accum, key) => {
    if (rollback[key]) {
      return accum.set(key, rollback[key](initialEntities.getIn([key]), entities.getIn([key])));
    } else {
      // Default to just reverting to the initial state for that
      // entity (before the optimistic update)
      return accum.set(key, initialEntities.getIn([key]));
    }
  }, Map());
};
