// @flow
import { Map, fromJS } from 'immutable';
import type { Map as MapType } from 'immutable';

import * as actionTypes from '../constants/action-types';

import type { Action } from '../actions';

export type State = MapType<string, MapType<string, any>>;

const initialState = Map();

const queries = (state: State = initialState, action: Action): State => {
  switch (action.type) {
    case actionTypes.RESET: {
      return Map();
    }
    case actionTypes.MUTATE_START:
    case actionTypes.REQUEST_START: {
      const { queryKey } = action;
      return state.deleteIn([queryKey]);
    }
    case actionTypes.MUTATE_FAILURE:
    case actionTypes.REQUEST_FAILURE: {
      const { queryKey } = action;

      return state.mergeIn(
        [queryKey],
        Map({
          responseBody: fromJS(action.responseBody),
          responseText: action.responseText,
          responseHeaders: fromJS(action.responseHeaders),
        }),
      );
    }
    default: {
      return state;
    }
  }
};

export default queries;
